@@ -6,8 +6,8 @@
 ;; Maintainer: Daniel Nouri <daniel.nouri@gmail.com>
 ;; URL: https://github.com/dnouri/pi-coding-agent
 ;; Keywords: ai llm ai-pair-programming tools
-;; Package-Version: 20260213.2143
-;; Package-Revision: f5af70c7ac24
+;; Package-Version: 20260218.1332
+;; Package-Revision: 531777c61cb3
 ;; Package-Requires: ((emacs "28.1") (markdown-mode "2.6") (transient "0.9.0"))
 
 ;; SPDX-License-Identifier: GPL-3.0-or-later
@@ -42,7 +42,7 @@
 ;;     Install from: https://github.com/misohena/phscroll
 ;;
 ;; Usage:
-;;   M-x pi           Start a session in current project
+;;   M-x pi           DWIM: hide, reuse, or create session
 ;;   C-u M-x pi       Start a named session
 ;;
 ;; Key Bindings:
@@ -130,24 +130,54 @@ Returns the chat buffer."
 ;;;###autoload
 (defun pi-coding-agent (&optional session)
   "Start or switch to pi coding agent session in current project.
-With prefix arg, prompt for SESSION name to allow multiple sessions.
-If already in a pi buffer and no SESSION specified, redisplays current session."
+Without prefix arg, DWIM behavior:
+  - From a pi buffer: hide this session (toggle off).
+  - Existing session for project: switch to it.
+  - No session: create a new one.
+With prefix arg, prompt for SESSION name to create a named session."
   (interactive
    (list (when current-prefix-arg
            (read-string "Session name: "))))
   (pi-coding-agent--check-dependencies)
-  (let (chat-buf input-buf)
-    (if (and (derived-mode-p 'pi-coding-agent-chat-mode 'pi-coding-agent-input-mode)
-             (not session))
-        ;; Already in pi buffer with no new session requested - use current session
-        (setq chat-buf (pi-coding-agent--get-chat-buffer)
-              input-buf (pi-coding-agent--get-input-buffer))
-      ;; Find or create session for current directory
-      (let ((dir (pi-coding-agent--session-directory)))
-        (setq chat-buf (pi-coding-agent--setup-session dir session))
-        (setq input-buf (buffer-local-value 'pi-coding-agent--input-buffer chat-buf))))
-    ;; Display and focus
-    (pi-coding-agent--display-buffers chat-buf input-buf)))
+  (let ((existing (and (not session)
+                       (car (pi-coding-agent-project-buffers)))))
+    (cond
+     ;; In a pi buffer with no session arg: hide this session
+     ((and (not session)
+           (derived-mode-p 'pi-coding-agent-chat-mode 'pi-coding-agent-input-mode))
+      (pi-coding-agent--hide-session-windows))
+     ;; Existing session for this project (no session arg): reuse it
+     (existing
+      (let ((input-buf (buffer-local-value
+                        'pi-coding-agent--input-buffer existing)))
+        (pi-coding-agent--display-buffers existing input-buf)))
+     ;; Otherwise: create or find named session
+     (t
+      (let* ((dir (pi-coding-agent--session-directory))
+             (chat-buf (pi-coding-agent--setup-session dir session))
+             (input-buf (buffer-local-value 'pi-coding-agent--input-buffer chat-buf)))
+        (pi-coding-agent--display-buffers chat-buf input-buf))))))
+
+;;;###autoload
+(defun pi-coding-agent-toggle ()
+  "Toggle pi coding agent window visibility for the current project.
+If pi windows are visible, hide them.  If hidden but a session
+exists, show them.  If no session exists, signal an error."
+  (interactive)
+  (pi-coding-agent--check-dependencies)
+  (let ((chat-buf (car (pi-coding-agent-project-buffers))))
+    (cond
+     ;; No session at all
+     ((null chat-buf)
+      (user-error "No pi session for this project"))
+     ;; Session visible: hide it
+     ((get-buffer-window-list chat-buf nil t)
+      (with-current-buffer chat-buf
+        (pi-coding-agent--hide-session-windows)))
+     ;; Session hidden: show it
+     (t
+      (let ((input-buf (buffer-local-value 'pi-coding-agent--input-buffer chat-buf)))
+        (pi-coding-agent--display-buffers chat-buf input-buf))))))
 
 ;;;; Performance Optimizations
 
